@@ -1,7 +1,8 @@
 import tkinter as tk
 from tkinter import ttk
-from typing import Optional, Any, List, Callable, Tuple, Union
+from typing import Dict, Optional, Any, List, Callable, Tuple, Union
 from pynput import keyboard
+from PIL import Image, ImageTk
 
 
 class ControlPanel(tk.Frame):
@@ -16,6 +17,7 @@ class ControlPanel(tk.Frame):
         self.master = master
         self.title = title
         self.grid()
+        self.rows:Dict[int, int] = {}
 
         if not exit_callback: exit_callback = self.on_closing
         self.master.protocol("WM_DELETE_WINDOW", exit_callback)
@@ -23,10 +25,14 @@ class ControlPanel(tk.Frame):
 
         self.widgets = {}
         self.containers = {}
+        self.images = {}
 
 
-    def __set_widget_position(self, widget:tk.Widget):
-        widget.grid(sticky=tk.NSEW)
+    def __set_widget_position(self, widget:tk.Widget, column:int=0):
+        if column not in self.rows:
+            self.rows[column] = 0
+        widget.grid(column=column, row=self.rows[column], sticky=tk.NSEW)
+        self.rows[column] += 1
 
         for container in self.containers.values():
             if container['open']:
@@ -60,25 +66,25 @@ class ControlPanel(tk.Frame):
                     binding[0] = False
 
         return super().update()
+        
 
-
-    def create_button(self, name:str, callback:Optional[Callable[[],None]]=None, hotkey:str=None):
+    def create_button(self, name:str, callback:Optional[Callable[[],None]]=None, hotkey:str=None, column:int=0):
 
         self.widgets[name] = tk.Button(self, command=callback)
         self.widgets[name]["text"] = name
 
-        self.__set_widget_position(self.widgets[name])
+        self.__set_widget_position(self.widgets[name], column)
 
         if hotkey is not None:
             self.__add_hotkey(hotkey, self.widgets[name])
             self.widgets[name]["text"] = f'{name} ({hotkey})'
 
 
-    def create_check_button(self, name:str, default:bool=False, callback:Optional[Callable[[bool],None]]=None, hotkey:str=None):
+    def create_check_button(self, name:str, default:bool=False, callback:Optional[Callable[[bool],None]]=None, hotkey:str=None, column:int=0):
 
         self.widgets[name] = ttk.Checkbutton(self)
         self.widgets[name]["text"] = name
-        self.__set_widget_position(self.widgets[name])
+        self.__set_widget_position(self.widgets[name], column)
 
         self.widgets[name].invoke()
         if not default: self.widgets[name].invoke()
@@ -91,26 +97,26 @@ class ControlPanel(tk.Frame):
             self.widgets[name]["text"] = f'{name} ({hotkey})'
 
 
-    def create_menu(self, name:str, choices:List[Any], callback:Optional[Callable[[Any],None]]=None):
+    def create_menu(self, name:str, choices:List[Any], callback:Optional[Callable[[Any],None]]=None, column:int=0):
 
         self.widgets[f'{name}_variable'] = tk.StringVar(self)
         self.widgets[f'{name}_variable'].set(choices[0]) # default value
 
-        self.__set_widget_position(tk.Label(self, text=name))
+        self.__set_widget_position(tk.Label(self, text=name), column)
 
         self.widgets[name] = tk.OptionMenu(self, self.widgets[f'{name}_variable'], *choices, command=callback)
 
-        self.__set_widget_position(self.widgets[name])
+        self.__set_widget_position(self.widgets[name], column)
 
     
-    def create_slider(self, name:str, start:float=0.0, end:float=1.0, step:float=0.01, default:float=None, show_value:bool=True, callback:Optional[Callable[[Any],None]]=None):
+    def create_slider(self, name:str, start:float=0.0, end:float=1.0, step:float=0.01, default:float=None, show_value:bool=True, callback:Optional[Callable[[Any],None]]=None, column:int=0):
         
         self.widgets[name] = tk.Scale(self, label=name, from_=start, to=end, resolution=step, orient=tk.HORIZONTAL, showvalue=show_value, command=callback)
 
         default = start if default is None else default
         self.widgets[name].set(default)
 
-        self.__set_widget_position(self.widgets[name])
+        self.__set_widget_position(self.widgets[name], column)
 
 
     def start_container(self, name:str, default:bool=False):
@@ -174,6 +180,16 @@ class ControlPanel(tk.Frame):
             raise NotImplementedError
 
 
+    def draw_image(self, name:str, image, column:int=0):
+        image_pil = Image.fromarray(image, mode="RGB")
+        image_tk = ImageTk.PhotoImage(image_pil)
+        if name not in self.images:
+            dummy_label = tk.Label(self, image=image_tk)
+            self.__set_widget_position(dummy_label, column)
+            self.images[name] = dummy_label
+        self.images[name].configure(image=image_tk)
+        self.images[name].image = image_tk
+
 
     def attach(self, other:'ControlPanel'):
         other.parent = self
